######################################
# File with Functions and Structures #
######################################

# Structure for parameters
mutable struct ParamsStruc{T <: Real, I <: Integer}
    # Demand curve
    α::T        # Constant
    β::T        # Slope
    # Cost function
    γ::T        # Quadratic term
    κ::T        # Cost reduction term
    # Discount factor
    δ::T
    # Time horizon for simulation
    T::I        # Number of periods (don't set too large)
    # Steady-state level of output
    ostar::T
end

################################################################################

# Set parameter values
function SetParams(; α = 1.0, β = 0.5,
                    γ = 0.5, κ = 0.9,
                    δ = 0.96, T = 10)
    # Steady-state level of output
    ostar = α/( β + γ*(1.0-κ)*(1.0-δ*κ) )
    # Output
    ParamsStruc(α,β,γ,κ,δ,T,ostar)
end

################################################################################

# Solves for the terminal level of output by solving the 2nd order difference equation
# by making a guess for the second starting value for output
function MultiShooting_o(o2,d,e,f,Params,flag)
    # Unpack parameters
    @unpack T,ostar = Params
    # Initialize vector of output
    ovec_MS = zeros(T)
    # Guess for second-period output
    ovec_MS[2] = o2

    # Iterate on difference equation starting from period 3 using starting
    # values for the first two periods for output
    for t=3:T
        ovec_MS[t] = d + e*ovec_MS[t-1] + f*ovec_MS[t-2]
        # Note that the starting value for output, ovec_MS[1] = 0
        # Likewise, the second starting value, ovec_MS[2], is equal to the guess
    end

    # Difference between terminal level of output and steady state level of output
    zero = ostar - ovec_MS[T]
    # Output
    if flag==1
        return zero
    else
        return ovec_MS,abs(zero)
    end
end

################################################################################

# Iterates on the 2nd-order difference equation for output using a guess path
# to insert a value for the future level of output. The guess path, called guesspath,
# is an input into the function
function ExtendPath_o(guesspath,g,h,i,Params)
    # Unpack parameters
    @unpack T,ostar = Params
    # Starting level of output is zero
    revisedpath = zeros(T)
    # Terminal output is the steady state level
    revisedpath[T] = ostar

    # Iterate on difference equation starting from period 2 using starting value
    # for the first period. There is no need to solve for the last period output
    # level since this taken to be the steady state level of output
    for t=2:T-1
        revisedpath[t] = g + h*revisedpath[t-1] + i*guesspath[t+1]
        # Note that starting value for output or revisedpath[1] = 0
        # Likewise, the terminal value, or revisedpath[T], is equal to ostar.
    end
    # Output
    return revisedpath
end

################################################################################

# Function to iteration on the guess of the path of output
function ExtendPathIteration(guessPath,g,h,i,Params)
    # Function to be solved for inside loop
    ExtendPath(x) = ExtendPath_o(x,g,h,i,Params)
    # Initialize iteration
    error_EP = 1.0
    iteration_EP = 1
    ovec_EP = zeros(Params.T)
    # Loop
    while iteration_EP <= 100  && error_EP >= 1e-9
        # Compute output path
        ovec_EP = ExtendPath(guessPath)
        # Residual
        error_EP = norm(ovec_EP .- guessPath)
        # Update path
        guessPath = ovec_EP
        # Update iteration
        iteration_EP = iteration_EP + 1
    end
    # Output
    return ovec_EP,error_EP,iteration_EP
end

################################################################################

function MakeCombinedFig(ovec,pvec,ovec_MS,pvec_MS,ovec_EP,pvec_EP,Params)
    # Set up vector for time
    time = (0:Params.T-1)
    # Decision rule approach
    # Output over time
    p1 = plot(time, ovec,
        title = "Output (decision rule approach)", titlefontsize=12,
        xlabel="Time", xtickfontsize=8, xguidefontsize=8,
        ylabel="Output", ytickfontsize=8, yguidefontsize=8,
        lw=3, legend=false)
    # Price over time
    p2 = plot(time, pvec,
        title = "Price (decision rule approach)", titlefontsize=12,
        xlabel="Time", xtickfontsize=8, xguidefontsize=8,
        ylabel="Price", ytickfontsize=8, yguidefontsize=8,
        lw=3, legend=false)

    # Multiple shooting
    # Output over time
    p3 = plot(time, ovec_MS,
        title = "Output (multiple shooting)", titlefontsize=12,
        xlabel="Time", xtickfontsize=8, xguidefontsize=8,
        ylabel="Output", ytickfontsize=8, yguidefontsize=8,
        lw=3, legend=false)
    # Price over time
    p4 = plot(time, pvec_MS,
        title = "Price (multiple shooting)", titlefontsize=12,
        xlabel="Time", xtickfontsize=8, xguidefontsize=8,
        ylabel="Price", ytickfontsize=8, yguidefontsize=8,
        lw=3, legend=false)

    # Extended path
    # Output over time
    p5 = plot(time, ovec_EP,
        title = "Output (extended path)", titlefontsize=12,
        xlabel="Time", xtickfontsize=8, xguidefontsize=8,
        ylabel="Output", ytickfontsize=8, yguidefontsize=8,
        lw=3, legend=false)
    # Price over time
    p6 = plot(time, pvec_EP,
        title = "Price (extended path)", titlefontsize=12,
        xlabel="Time", xtickfontsize=8, xguidefontsize=8,
        ylabel="Price", ytickfontsize=8, yguidefontsize=8,
        lw=3, legend=false)

    # Combined plot
    p = plot(p1, p2, p3, p4, p5, p6, layout = (3, 2), legend = false)
    # Save figure
    savefig("Figures/fig_comparing_methods.pdf")
    return p
end

################################################################################
