#########################
# Main file to run code #
#########################

# Housekeeping
exit()
clearconsole()

# Load packages
using LinearAlgebra, Parameters, Plots, Polynomials, Roots

# Load functions
include("Functions/f_Functions.jl")

# Call exogenous parameters
Params = SetParams()

## Method 1: Solve model taking the Decision-Rule Approach

# Set up the Quadratic Formulae for Psi
a = -Params.δ*Params.γ*Params.κ                             # Coefficient on squared term
b = Params.γ + Params.β + Params.δ*Params.γ*Params.κ^2.0    # Coefficient on linear term
c = -Params.γ*Params.κ                                      # Constant term
# Polynomial (a1 + a2*x + a3*x^2)
p = [c,b,a]

# Grid for ψ
ψ_grid = range(0.0,step=0.1,stop=1.0)
# Values of quadratic
quad4_ψ = a*ψ_grid.^2 .+ b*ψ_grid .+ c

# Plot the quadratic equation for psi
plot(ψ_grid, quad4_ψ,
    title = "The quadratic for ψ", titlefontsize=16,
    xlabel="ψ", xtickfontsize=12,
    ylabel="Quadratic", ytickfontsize=12, yguidefontsize=12,
    lw=3, legend=false)
# Save figure
savefig("Figures/fig_QuadraticEq.pdf")

# Find roots of quadratic and take the lowest value
ψ = minimum( roots(Polynomial(p)) )         # Slope term on law of motion for output
η = (1.0-ψ)*Params.ostar                    # Constant term on law of motion for output

# Set up vector to store outputs for decision rule approach
# Note the the first element in ovec[1] actually corresponds to period zero
ovec = zeros(Params.T)

# Iterate on difference equation for output
for t = 2:Params.T
    # Starting value for loop is ovec(1,1) = 0
    ovec[t] = η + ψ*ovec[t-1]
end
# Compute prices over time
pvec = Params.α .- Params.β.*ovec./2.0

## Method 2: Solve model using multiple shooting

# Set up terms for second-order difference equation
d = -Params.α/(-a)      # Constant term
e = b/(-a)              # Coefficient on lagged output
f = c/(-a)              # Coefficient on lagged output from two periods ago

# Guess for output vector
o2guess = 0.5*ovec[2]
# Function to be solved for
MultiShooting(x) = MultiShooting_o(x,d,e,f,Params,1)
# Solve for output in period 2
o2 = find_zero(MultiShooting, o2guess)
# Recover output vector and error
ovec_MS,error_MS = MultiShooting_o(o2,d,e,f,Params,0)

# Check solution
if error_MS >= 1e-9
    println("Solution has not been found")
    println("Error = $error_MS")
end

# Compute prices over time
pvec_MS = Params.α .- Params.β.*ovec_MS./2.0

## Method 3: Solve model using extended-path method

# Set up terms for second-order difference equation
g = Params.α/b               # Constant term
h = Params.γ*Params.κ/b     # Coefficient on lagged output
i = -a/b                    # Coefficient on future output

# Guess of path of output (use output from multiple shooting)
guessPath = range(0.0, step=Params.ostar/(Params.T-1), stop=Params.ostar)
# Function in which path is iterated
ovec_EP,error_EP,iteration_EP = ExtendPathIteration(guessPath,g,h,i,Params)
# Print stuff
println("Number of iterations for extended path method = ", (iteration_EP-1))

# Compute prices over time
pvec_EP = Params.α .- Params.β.*ovec_EP./2.0

## Final plot

# Here a graph with 6 panels is constructed in the form of a 3 by 2 matrix
p1 = MakeCombinedFig(ovec,pvec,ovec_MS,pvec_MS,ovec_EP,pvec_EP,Params)
